//------------------------------------------------------------------------
// GraphDraw class
// This class draws a unit sound wave and makes it possible to adjust
// the graph's phase, amplitude and frequency. If user clicks on the
// 'Combine' button, it combines two graphs and draws a new graph.
// The phase adjustment changes PI / 4 of a graph's period.
//
// INPUT: user mouse clicking on the button.
//
// OUTPUT: a sound graph reflecting the control buttons.
//
// @author Jungkee Kim
// @version 1.1, May 1997

import java.applet.*;
import java.awt.*;
import java.lang.Math;
import java.util.*;

class GraphDraw extends Panel {
  GraphWaves            parent;                       //parent class
  GraphDrawCanvas	paper;                        //graph canvas
  GraphDrawPanel	board;                        //control panel

  GraphDraw(GraphWaves parent) {
    this.parent = parent;
    setLayout(new BorderLayout()); 
    paper = new GraphDrawCanvas(this);
    board = new GraphDrawPanel(this);
    add("Center", paper);
    add("South", board);
    paper.setBackground(Color.black);
    paper.setForeground(Color.green);
    board.setBackground(new Color(0xFFEBCD));
  }

  public void setSine() {
    paper.setSine();
  }
  
  public void setRectangular() {
    paper.setRectangular();
  }
  
  public void setTriangle() {
    paper.setTriangle();
  }
  
  public void paint (Graphics g) {
    paper.paint(g);
    board.paint(g);
  }
  
  public void start() {
    paper.start();
    board.start();
  }

  public void stop() {
    paper.stop();
  }

  public void update(Graphics g) {
    paper.paint(g);
  }
}

/* This class draws a graph on the canvas. Three kinds of graph can be drawn */
class GraphDrawCanvas extends Canvas implements Runnable { 
  
  GraphDraw parent;
  int shape;
  Thread thread;				//Thread variable 
  double dx = 0.0;                              //delta x for phase change
  double dy = 1.0;                              //delta y for amplitude change
  double df = 1.0;                              //delta f for frequency change
  
  public GraphDrawCanvas(GraphDraw parent) {
    this.parent = parent;
    thread = new Thread(this);
  } 
  
  public void setSine() {
    shape = 1;
  }
  
  public void setRectangular() {
    shape = 2;
  }
  
  public void setTriangle() {
    shape = 3;
  }
  
  public void start() { 
    
    if (thread.isAlive()) {		//Checking the life of thread 
      thread.resume();		        //If other page reference, 
    }				        // resume the thread. 
    
    else {				//Start thread from begining 
      thread.start(); 
    } 
  } 
  
  public void stop() {                  //Stop thread. 
    thread.suspend(); 
  } 
  
  public void destroy() {		//Destroy thread. 
    thread.stop(); 
  } 
  
  public void run() { 
    repaint(); 
  } 
  
  /* This method generates a function value at the particular point */
  double f(double x) { 
    switch (shape) {
    /* Sine wave function */
    case 1 :
      return (Math.sin(x*df/5 +dx) * dy + 2 )* size().height / 4;
    /* Rectangular wave function */
    case 2:
      double y = x * df / 5 + dx;
      if (Math.floor(y / Math.PI) % 2 == 0) {       //upper part, but
	return (1 * dy + 2) * size().height / 4;    // displayed as lower part 
      }
      else {                                        //lower part, but
	return (-1 * dy + 2) * size().height / 4;   // displayed as upper part
      }
    /* Triangle wave function */
    case 3:
      y = x * df / 5 + dx;
      double length = Math.PI / 2 * 5 / df;
      double height = dy * size().height / 4;
      double slope = height / length;
      double period = 4 * length;
      double d = dx * 2 / Math.PI * length;
      /* Triangle wave can be divided as 3 part line functions */
      if ( d < 0 ) { x += period; }
      if (Math.floor(y / Math.PI * 2) % 4 == 0) {
	return slope * ( (x+d) % period ) + size().height / 2;
      }
      else if (Math.floor(y / Math.PI * 2) % 4 == 3) {
	return slope * (( (x+d) - 3 * length) % period) + size().height / 2 - height;
      }
      else {
	return -1 * slope * (( x+d - length ) % period) + size().height / 2 + height;
      }
    default:
      return 0;
    }
  }
  
  public void paint(Graphics g) { 
    for (int x = 0 ; x < size().width ; x++) { 
      g.drawLine(x, (int)f(x), x + 1, (int)f(x + 1)); 
    } 
  } 
  
  /* adjust phase responding button action */
  public void update_phase(int flag) { 
    switch( flag ) { 
    case 1 : dx += Math.PI / 4; 
      break; 
    case 2 : dx -= Math.PI / 4;; 
      break; 
    default : break; 
    }
    if ( dx < 0 ) { dx += Math.PI * 2; } 
    repaint(); 
  }
  
  /* adjust amplitude responding button action */
  public void update_amplitude(int flag) {
    switch( flag ) {
    case 1 : dy -= 0.1;
      break;
    case 2 : dy += 0.1;
      break;
    default : break;
    }
    if ( dy < 0 ) {
      dy = 0;
    }
    if ( dy > 2 ) {
      dy = 2;
    }
    repaint();
  }

  /* adjust frequency responding button action */
  public void update_frequency(int flag) {
    switch( flag ) {
    case 1 : df -= 0.1;
      break;
    case 2 : df += 0.1;
      break;
    default : break;
    }
    if ( df < 0 ) {
      df = 0;
    }
    repaint();
  }
  
} 

/* This class shows adjustment buttons */
class GraphDrawPanel extends Panel { 
  GraphDraw parent; 
  
  GraphDrawPanel(GraphDraw parent) { 
    this.parent = parent; 
    setLayout(new GridLayout(1, 9, 2, 2)); 
    
    add(new Button("<"));
    add(new Label("Pha."));
    add(new Button(">"));
    add(new Button("-"));
    add(new Label("Amp."));
    add(new Button("+"));
    add(new Button("L"));
    add(new Label("Freq."));
    add(new Button("H"));
  } 
  
  public void start() { 
  } 
  
  public void paint(Graphics g) { 
  } 
  
  public boolean action(Event evt, Object arg) { 
    if (evt.target instanceof Button) 
      changePhase((String)arg); 
    return true; 
  } 
  
  void changePhase(String bname) { 
    if (bname.equals("<")) parent.paper.update_phase(1); 
    else if (bname.equals(">")) parent.paper.update_phase(2);
    else if (bname.equals("-")) parent.paper.update_amplitude(1);
    else if (bname.equals("+")) parent.paper.update_amplitude(2);
    else if (bname.equals("L")) parent.paper.update_frequency(1);
    else if (bname.equals("H")) parent.paper.update_frequency(2);
  } 
}
