//------------------------------------------------------------------------
// GraphDraw class
// This class draws a unit sound wave and makes it possible to adjust
// the graph's phase, amplitude and frequency. If user clicks on the
// 'Combine' button, it combines two graphs and draws a new graph.
// The phase adjustment changes PI / 4 of a graph's period.
//
// INPUT: user mouse clicking on the button.
//
// OUTPUT: a sound graph reflecting the control buttons.
//
// @author Jungkee Kim

import java.applet.*;
import java.awt.*;
import java.lang.Math;
import java.util.*;

import messages.*;
import consts.Const;

class GraphDraw extends Panel {
  GraphWaves            parent;                       //parent class
  GraphDrawCanvas	paper;                        //graph canvas
  GraphDrawPanel	board;                        //control panel

  GraphDraw(GraphWaves parent) {
    this.parent = parent;
    setLayout(new BorderLayout()); 
    paper = new GraphDrawCanvas(this);
    board = new GraphDrawPanel(this);
    add("Center", paper);
    add("South", board);
    paper.setBackground(Color.black);
    paper.setForeground(Color.red);
    board.setBackground(new Color(0xEBFFCD));
  }

  public void setSine() {
    paper.setSine();
  }
  
  public void setRectangular() {
    paper.setRectangular();
  }
  
  public void setTriangle() {
    paper.setTriangle();
  }
  
  public void paint (Graphics g) {
    paper.paint(g);
    board.paint(g);
  }
  
  public void start() {
    paper.start();
    board.start();
  }

  public void stop() {
    paper.stop();
  }

  public void update(Graphics g) {
    paper.paint(g);
  }
}

/* This class draws a graph on the canvas. Three kinds of graph can be drawn */
class GraphDrawCanvas extends Canvas implements Runnable { 
  
  GraphDraw parent;
  int shape;
  Thread thread;				//Thread variable 
  double dx = 0.0;                              //delta x for phase change
  double dy = 1.0;                              //delta y for amplitude change
  double df = 1.0;                              //delta f for frequency change
  double cf = 5;				//constant f for frequency
  String freq = "255 Hz";			//Current frequency

  public GraphDrawCanvas(GraphDraw parent) {
    this.parent = parent;
    thread = new Thread(this);
  } 
  
  public void setSine() {
    shape = 1;
  }
  
  public void setRectangular() {
    shape = 2;
  }
  
  public void setTriangle() {
    shape = 3;
  }
  
  public void start() { 
    
    if (thread.isAlive()) {		//Checking the life of thread 
      thread.resume();		        //If other page reference, 
    }				        // resume the thread. 
    
    else {				//Start thread from begining 
      thread.start(); 
    } 
  } 
  
  public void stop() {                  //Stop thread. 
    thread.suspend(); 
  } 
  
  public void destroy() {		//Destroy thread. 
    thread.stop(); 
  } 
  
  public void run() { 
    repaint(); 
  } 
  
  /* This method generates a function value at the particular point */
  double f(double x) { 
    switch (shape) {
    /* Sine wave function */
    case 1 :
      return (Math.sin(x*df/cf +dx) * dy + 2 )* size().height / 4;
    /* Rectangular wave function */
    case 2:
      double y = x * df / cf + dx;
      if (Math.floor(y / Math.PI) % 2 == 0) {       //upper part, but
	return (1 * dy + 2) * size().height / 4;    // displayed as lower part 
      }
      else {                                        //lower part, but
	return (-1 * dy + 2) * size().height / 4;   // displayed as upper part
      }
    /* Triangle wave function */
    case 3:
      y = x * df / cf + dx;
      double length = Math.PI / 2 * 5 / df;
      double height = dy * size().height / 4;
      double slope = height / length;
      double period = 4 * length;
      double d = dx * 2 / Math.PI * length;
      /* Triangle wave can be divided as 3 part line functions */
      if ( d < 0 ) { x += period; }
      if (Math.floor(y / Math.PI * 2) % 4 == 0) {
	return slope * ( (x+d) % period ) + size().height / 2;
      }
      else if (Math.floor(y / Math.PI * 2) % 4 == 3) {
	return slope * (( (x+d) - 3 * length) % period) + size().height / 2 - height;
      }
      else {
	return -1 * slope * (( x+d - length ) % period) + size().height / 2 + height;
      }
    default:
      return 0;
    }
  }
  
  public void paint(Graphics g) { 
    for (int x = 0 ; x < size().width ; x++) { 
      g.drawLine(x, (int)f(x), x + 1, (int)f(x + 1)); 
    }
    g.drawString(freq, 5, 10);
  } 
  
  /* adjust phase responding button action */
  public void update_phase(int flag) { 
    switch( flag ) { 
    case 1 : dx += Math.PI / 4; 
      break; 
    case 2 : dx -= Math.PI / 4;; 
      break; 
    default : break; 
    }
    if ( dx < 0 ) { dx += Math.PI * 2; } 
    repaint(); 
  }
  
  /* adjust amplitude responding button action */
  public void update_amplitude(int flag) {
    switch( flag ) {
    case 1 : dy -= 0.1;
      break;
    case 2 : dy += 0.1;
      break;
    default : break;
    }
    if ( dy < 0 ) {
      dy = 0;
    }
    if ( dy > 2 ) {
      dy = 2;
    }
    repaint();
  }

  /* adjust frequency responding button action */
  public void update_frequency(int flag) {
    switch( flag ) {
    case 1 : df -= 0.1;
      break;
    case 2 : df += 0.1;
      break;
    default : break;
    }
    if ( df < 0 ) {
      df = 0;
    }
    repaint();
  }
  
  /* Set frequency responding button action */
  public void set_frequency(int flag) {
    switch( flag ) {
    case 1 : cf = 8000 / 2 / Math.PI / 260;
      freq = "260 Hz (Do)";
      break;
    case 2 : cf = 8000 / 2 / Math.PI / 293;
      freq = "293 Hz (Re)";
      break;
    case 3 : cf = 8000 / 2 / Math.PI / 325;
      freq = "325 Hz (Mi)";
      break;
    case 4 : cf = 8000 / 2 / Math.PI / 340;
      freq = "340 Hz (Fa)";
      break;
    case 5 : cf = 8000 / 2 / Math.PI / 390;
      freq = "390 Hz (Sol)";
      break;
    case 6 : cf = 8000 / 2 / Math.PI / 440;
      freq = "440 Hz (La)";
      break;
    case 7 : cf = 8000 / 2 / Math.PI / 488;
      freq = "488 Hz (Si)";
      break;
    default : cf = 5; freq ="255 Hz"; break;
    }
    repaint();
  }
  
} 

/* This class shows adjustment buttons */
class GraphDrawPanel extends Panel { 
  GraphDraw parent; 
  
  GraphDrawPanel(GraphDraw parent) { 
    this.parent = parent; 
    setLayout(new GridLayout(1, 9, 2, 2)); 
    
    add(new Button("-"));
    add(new Button("C"));
    add(new Button("D"));
    add(new Button("E"));
    add(new Button("F"));
    add(new Button("G"));
    add(new Button("A"));
    add(new Button("B"));
    add(new Button("+"));
  } 
  
  public void start() { 
  } 
  
  public void paint(Graphics g) { 
  } 
  
  public boolean action(Event evt, Object arg) { 
    if (evt.target instanceof Button) 
      changePhase((String)arg); 
    return true; 
  } 
  
  void changePhase(String bname) { 
    byte[] act = new byte[2];
    act[0] = 0;
    act[1] = 0;

    if (bname.equals("C")) {
      act[1] = 1;
      AppEventMessage em = new AppEventMessage(Const.DATA, act);
      parent.parent.send(em);
      parent.paper.set_frequency(1);
    }
    else if (bname.equals("D")) {
      act[1] = 2;
      AppEventMessage em = new AppEventMessage(Const.DATA, act);
      parent.parent.send(em);
      parent.paper.set_frequency(2);
    }
    else if (bname.equals("E")) {
      act[1] = 3;
      AppEventMessage em = new AppEventMessage(Const.DATA, act);
      parent.parent.send(em);
      parent.paper.set_frequency(3);
    }
    else if (bname.equals("F")) {
      act[1] = 4;
      AppEventMessage em = new AppEventMessage(Const.DATA, act);
      parent.parent.send(em);
      parent.paper.set_frequency(4);
    }
    else if (bname.equals("G")) {
      act[1] = 5;
      AppEventMessage em = new AppEventMessage(Const.DATA, act);
      parent.parent.send(em);
      parent.paper.set_frequency(5);
    }
    else if (bname.equals("A")) {
      act[1] = 6;
      AppEventMessage em = new AppEventMessage(Const.DATA, act);
      parent.parent.send(em);
      parent.paper.set_frequency(6);
    }
    else if (bname.equals("B")) {
      act[1] = 7;
      AppEventMessage em = new AppEventMessage(Const.DATA, act);
      parent.parent.send(em);
      parent.paper.set_frequency(7);
    }
    else if (bname.equals("-")) {
      act[0] = 2;
      AppEventMessage em = new AppEventMessage(Const.DATA, act);
      parent.parent.send(em);
      parent.paper.update_amplitude(1);
    }
    else if (bname.equals("+")) {
      act[0] = 3;
      AppEventMessage em = new AppEventMessage(Const.DATA, act);
      parent.parent.send(em);
      parent.paper.update_amplitude(2);
    }
  } 
}
